<?php

/*
 * This file is part of the Serenity package.
 *
 * (c) CP Creation <web@cpcreation.fr>
 *
 * For the full copyright and license information, please view the LICENSE
 * file that was distributed with this source code.
 */

namespace Serenity\CoreBundle\Notification\Enum;

enum PushLevel: string
{
    case SUCCESS = 'success';
    case LOGIN_SUCCESS = 'login-success';
    case WARNING = 'warning';
    case ERROR = 'error';
    case LOGIN_ERROR = 'login-error';
    case INFO = 'info';
}
