<?php

namespace Serenity\CoreBundle\Shared\Menu;

use Serenity\CoreBundle\Shared\Feature\NestedTreeInterface;

interface MenuTreeManipulatorInterface
{
    public function insertLastChild(NestedTreeInterface $new, NestedTreeInterface $parent): void;
    public function insertBefore(NestedTreeInterface $new, NestedTreeInterface $ref): void;
    public function insertAfter(NestedTreeInterface $new, NestedTreeInterface $ref): void;
}
