<?php

/*
 * This file is part of the Serenity package.
 *
 * (c) CP Creation <web@cpcreation.fr>
 *
 * For the full copyright and license information, please view the LICENSE
 * file that was distributed with this source code.
 */

namespace Serenity\CoreBundle\Shared\Feature;

use Serenity\CoreBundle\Shared\Common\UserInterface;

trait OwnerTrait
{
    private ?UserInterface $owner = null;

    public function getOwner(): ?UserInterface
    {
        return $this->owner;
    }

    public function setOwner(?UserInterface $owner = null): void
    {
        $this->owner = $owner;
    }
}
