<?php

declare(strict_types=1);

/*
 * This file is part of the Serenity package.
 *
 * (c) CP Creation <web@cpcreation.fr>
 *
 * For the full copyright and license information, please view the LICENSE
 * file that was distributed with this source code.
 */

namespace Serenity\CoreBundle\Shared\Doctrine\Attribute;

#[\Attribute(\Attribute::TARGET_CLASS)]
readonly class RowLevelSecurity
{
    public function __construct(
        public string $name = '',
        public string $role = '',
        public string $for = '',
        public string $using = '',
        public bool $withCheck = false,
    ) {
    }
}
