{% extends "tailwind_2_layout.html.twig" %}
{%- block ckeditor_widget -%}
        {{ block('textarea_widget' ) }}
        <div class="text-sm italic ck-editor-char {{ not form.vars.valid ? 'is-text-error' : '' }}">
            Characters: <span
                data-char-count>0</span>{% if form.vars.attr['data-ckeditor-maxlength-value'] %} / {{ form.vars.attr['data-ckeditor-maxlength-value'] }} {% endif %}
        </div>
{%- endblock ckeditor_widget -%}
