<?php

/*
 * This file is part of the Serenity package.
 *
 * (c) CP Creation <web@cpcreation.fr>
 *
 * For the full copyright and license information, please view the LICENSE
 * file that was distributed with this source code.
 */

namespace Serenity\CoreBundle\Shared\EventListener;

use Symfony\Component\EventDispatcher\Attribute\AsEventListener;
use Symfony\Component\HttpKernel\Event\ControllerEvent;
use Symfony\Component\HttpKernel\Exception\AccessDeniedHttpException;
use Symfony\Component\HttpKernel\HttpKernelInterface;
use Symfony\Component\HttpKernel\KernelEvents;
use Symfony\Component\Security\Core\Authorization\AuthorizationCheckerInterface;

/**
 * Vérifie automatiquement les rôles définis dans la configuration des routes via l’attribut `_role`.
 *
 * Cela permet de sécuriser une route directement depuis son fichier YAML :
 *
 * ```yaml
 * admin_shop_product:
 *     resource: 'routes/admin/*.yaml'
 *     prefix: '/dashboard/shop'
 *     defaults:
 *         _role: ROLE_MODERATOR
 *     host: '%routing.host_admin%'
 * ```
 *
 * Si l’utilisateur n’a pas le rôle requis (`ROLE_MODERATOR` ici), une erreur 403 est levée.
 *
 * 🔒 Ce mécanisme vient en complément du système `access_control` de Symfony :
 * - `access_control` reste idéal pour les restrictions globales
 * - `_role` permet une gestion fine route par route
 */
#[AsEventListener(event: KernelEvents::CONTROLLER, priority: 90)]
readonly class AuthorizationCheckerRouteListener
{
    public function __construct(
        private AuthorizationCheckerInterface $authorizationChecker,
    ) {
    }

    public function __invoke(ControllerEvent $event): void
    {
        if (HttpKernelInterface::MAIN_REQUEST !== $event->getRequestType()) {
            return;
        }

        if (!$event->getRequest()->attributes->get('_role')) {
            return;
        }

        if (!$this->authorizationChecker->isGranted($event->getRequest()->attributes->get('_role'))) {
            throw new AccessDeniedHttpException();
        }
    }
}
