{% extends '@SerenityAdmin/layout/base.html.twig' %}

{% block settings %}
    <twig:SerenityAdmin:Settings>
        <twig:block name="settings_content">
            test
        </twig:block>
    </twig:SerenityAdmin:Settings>
{% endblock %}
