<?php

declare(strict_types=1);

/*
 * This file is part of the Serenity package.
 *
 * (c) CP Creation <web@cpcreation.fr>
 *
 * For the full copyright and license information, please view the LICENSE
 * file that was distributed with this source code.
 */

namespace Serenity\CoreBundle\Shared\Aggregate;

use Serenity\CoreBundle\Shared\Event\DomainEvent;

interface AggregateRootInterface
{
    public function recordEvent(DomainEvent $domainEvent): void;

    public function domainEventsEmpty(): bool;

    /**
     * @return array<DomainEvent>
     */
    public function pullDomainEvents(): array;
}
