<?php

/*
 * This file is part of the Serenity package.
 *
 * (c) CP Creation <web@cpcreation.fr>
 *
 * For the full copyright and license information, please view the LICENSE
 * file that was distributed with this source code.
 */

namespace Serenity\CoreBundle\Shared\Feature;

interface SerenitySaasInterface
{
    public function getSaasIdentifier(): ?string;

    public function setSaasIdentifier(?string $saasIdentifier = null): void;

    public function isSaasIdentifierAutoGenerated(): bool;

    public function setSaasIdentifierAutoGenerated(bool $saasIdentifierAutoGenerated): void;
}
