<?php

/*
 * This file is part of the Serenity package.
 *
 * (c) CP Creation <web@cpcreation.fr>
 *
 * For the full copyright and license information, please view the LICENSE
 * file that was distributed with this source code.
 */

namespace Serenity\CoreBundle\Shared\Serializer;

class CircularReferenceHandler
{
    public static function handle(object $object): string
    {
        return method_exists($object, 'getId')
            ? $object->getId()
            : spl_object_id($object);
    }
}
