<?php

/*
 * This file is part of the Serenity package.
 *
 * (c) CP Creation <web@cpcreation.fr>
 *
 * For the full copyright and license information, please view the LICENSE
 * file that was distributed with this source code.
 */

namespace Serenity\CoreBundle\Slug\Event;

use Symfony\Component\Uid\Uuid;

final readonly class SlugChanged
{
    public function __construct(
        public string $entityClass,
        public Uuid $entityId,
        public string $oldSlug,
    ) {
    }
}
