<?php

/*
 * This file is part of the Serenity package.
 *
 * (c) CP Creation <web@cpcreation.fr>
 *
 * For the full copyright and license information, please view the LICENSE
 * file that was distributed with this source code.
 */

namespace Serenity\CoreBundle\Shared\ObjectMapper;

use Symfony\Component\DependencyInjection\Attribute\AutoconfigureTag;
use Symfony\Component\ObjectMapper\ObjectMapperInterface;

#[AutoconfigureTag('serenity.object_mapper')]
interface StrategyObjectMapperInterface extends ObjectMapperInterface
{
    public function supports(string $source, string $target): bool;
}
