<?php

/*
 * This file is part of the Serenity package.
 *
 * (c) CP Creation <web@cpcreation.fr>
 *
 * For the full copyright and license information, please view the LICENSE
 * file that was distributed with this source code.
 */

namespace Serenity\CoreBundle\Notification\Notifier;

use Serenity\CoreBundle\Notification\Enum\PushChannel;
use Serenity\CoreBundle\Notification\Enum\PushLevel;
use Serenity\CoreBundle\Notification\Enum\PushTarget;
use Serenity\CoreBundle\Notification\Enum\PushTransport;
use Serenity\CoreBundle\Notification\Enum\PushType;
use Symfony\Component\DependencyInjection\Attribute\AutoconfigureTag;

#[AutoconfigureTag('serenity-core.push_notifier')]
interface NotifierInterface
{
    public function push(string $message, PushChannel $channel, PushTarget $target, PushLevel $level, PushTransport $transport, PushType $type, ?string $template = null): void;

    public static function supports(): ?PushTransport;
}
