{% form_theme form with admin_form_themes() only %}
<div {{ attributes }} >
    {{ form_start(form, {
        attr: {
            'data-action': 'live#action:prevent',
            'data-live-action-param': 'debounce(300)|files|save',
            'novalidate': 'true',

            'x-on:keydown.enter.prevent': "if ($event.target.form === $el) {
              $el.querySelector('button[type=submit]').click()
          }"
        }
    }) }}
    {{ form_row(form) }}
    {{ form_rest(form) }}
    {{ form_end(form) }}
</div>
