<?php

/*
 * This file is part of the Serenity package.
 *
 * (c) CP Creation <web@cpcreation.fr>
 *
 * For the full copyright and license information, please view the LICENSE
 * file that was distributed with this source code.
 */

namespace Serenity\CoreBundle\Seed\Exception;

use Symfony\Component\Messenger\Exception\UnrecoverableMessageHandlingException;

final class SeedAlreadyRunningException extends UnrecoverableMessageHandlingException
{
}
