<?php

/*
 * This file is part of the Serenity package.
 *
 * (c) CP Creation <web@cpcreation.fr>
 *
 * For the full copyright and license information, please view the LICENSE
 * file that was distributed with this source code.
 */

namespace Serenity\CoreBundle\Seed\Command;

use Serenity\CoreBundle\Seed\SeedPayload;
use Symfony\Component\Messenger\Attribute\AsMessage;

#[AsMessage('seed')]
final readonly class SeedCommand
{
    public function __construct(
        public SeedPayload $payload,
    ) {
    }
}
